% ViPrice Toolbox 0.2
% Copyright (C) 2015  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
classdef CNNDescriptorClass
    
    properties
        net
        descriptorLength
        id
    end
    
    methods
       function obj = CNNDescriptorClass(matconvnet_path)
          % path to CNN toolbox
          addpath(fullfile(matconvnet_path, 'matlab'));

          vl_setupnn;

          obj.net = load(fullfile(matconvnet_path, 'models', 'imagenet-vgg-m.mat'));

          % we use just layer conv3 --> we can cut the rest, this saves about half the
          % computational time
          obj.net.layers(11:end) = [];
          
          obj.descriptorLength = 13*13*512;
          obj.id = 'CNN';
          
       end
    
       % returns number of elements in the descriptor (length of the
       % row vector)
       function descriptorLength = getDescriptorLength(obj)
           descriptorLength = obj.descriptorLength;
       end
       
       % P ... image patch of arbitrary size (it is internally resized),
       %       range: [0, 255]
       % D ... (large) row vector with network response
       function D = getDescriptor(obj, P)
          % normalize
          im_ = single(P) ; % note: 255 range
          im_ = imresize(im_, obj.net.normalization.imageSize(1:2)) ;
          im_ = im_ - obj.net.normalization.averageImage ;
           
          % run the CNN      
          res = vl_simplenn(obj.net, im_);
          D = reshape(res(9+1).x, 1, numel(res(9+1).x));  
       end
       
       % return id as string
       function id = getId(obj)
           id = obj.id;
       end
   end
end