% ViPrice Toolbox 0.2
% Copyright (C) 2015  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
classdef LandmarksClass
    
    properties
        describer
        detector
    end
    
    methods
       % describer_ ... instance of descriptor class
       function obj = LandmarksClass(detector_, describer_)
           obj.describer = describer_;
           obj.detector = detector_;
       end            
        
       % Save landmarks L
       function save(obj, data_params, idx, L)
           featureSavePath = getPathOfFeature(data_params, obj, idx);
           save(featureSavePath, 'L', '-v7');
       end
       
       % Load landmarks L
       function L = load(obj, data_params, idx)
           featureSavePath = getPathOfFeature(data_params, obj, idx);
           load(featureSavePath, 'L');
       end
       
       % Call detector function to get location of the regions in the image
       % regions:
       %   .ROI ... [x1 y1 x2 y2]
       %   .mask ... optional binary mask
       %
       function regions = getRegions(obj, I)
           regions = obj.detector.getRegions(I);
       end
       
       % Compute descriptor for each region
       % D ... matrix, each row is a descriptor
       function D = getDescriptorsForRegions(obj, I, regions)
           
           D = zeros(numel(regions), obj.describer.getDescriptorLength(), 'single'); 
           for i=1:numel(regions)
               
               % get image patch
               ROI = regions{i}.ROI; 
               P = I(ROI(2):ROI(4),ROI(1):ROI(3), :);
               
               % compute descriptor
               D(i,:) = obj.describer.getDescriptor(P);
           end
       end
       
       % Compute landmarks for an image. First calls the detector then the
       % describer.
       % landmarks:
       %    .regions ... regions
       %    .D       ... matrix, each row is a descriptor
       function landmarks = getLandmarks(obj, I)
           landmarks.regions = obj.getRegions(I);
           landmarks.D = obj.getDescriptorsForRegions(I, landmarks.regions);
       end
       
       % Returns combined ids of describer and detector as string
       function id = getId(obj)
           id = [obj.describer.getId() '_' obj.detector.getId()];
       end
    end
end