% ViPrice Toolbox 0.2
% Copyright (C) 2015  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
classdef SiftDetectorClass 
    properties
        id
        params
    end
    
    methods
        
            
       function obj = SiftDetectorClass(params, vlFeat_path)
           % setup VlFeat
           addpath(vlFeat_path);
           vl_setup;    
           
           if ~exist('params', 'var') && ~isempty(params)
              if ~exist('params', 'var')
                  params.n = 50;
                  params.scale = 6;
                  params.nIter = 50;              
              end
           end
           params.n = params.n;
           
           % setup id for storage           
           obj.id = ['SIFT' num2str(params.scale)];
           obj.params = params;           
           
       end
        
       % Get location of the regions in the image
       % params:
       %   .n     ... wished number of regions (e.g. 50)
       %   .scale ... scale of the SIFT regions (e.g. 6)
       %   .nIter ... number of iterations to obtain wished number of
       %              regions
       % regions:
       %   .ROI ... [x1 y1 x2 y2]
       %   .mask ... optional binary mask
       %
       function regions = getRegions(obj, I)

          peakThresh = 6.0;
          dPeakThresh = 0.5;
          for i=1:obj.params.nIter

            frames = vl_sift(single(rgb2gray(I)), 'Octaves', 5, 'FirstOctave', 1, 'PeakThresh', peakThresh);

            if size(frames,2) > obj.params.n*1.1
              peakThresh = peakThresh + dPeakThresh;
            elseif size(frames,2) < obj.params.n*0.9
              peakThresh = peakThresh - dPeakThresh;
            else
              break;
            end
            dPeakThresh = max(0.1,dPeakThresh*0.9);
          end


          frames(3,:) = frames(3,:)*obj.params.scale;
          fprintf('detected %d regions \n', size(frames,2));

          regions = keypoints2regionsCNN(frames, I);   

       end
    
       % return id as string
       function id = getId(obj)
           id = obj.id;
       end
    end
end