% ViPrice Toolbox 0.2
% Copyright (C) 2015  Peer Neubert, peer.neubert@etit.tu-chemnitz.de
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% --------------------------------------------------
%
% Compute regions with patch and mask for keypoint frames
% 
% frames ... each column [x, y, scale, orientation]'
% I ... input image
% regions{i}.mask ... mask of i-th region
% regions{i}.ROI ... [x1,y1,x2, y2] location of mask and patch
% regions{i}.midpoint ... [x,y]
%
function regions = keypoints2regionsCNN(frames, I)

  regions = {};
  
  % create bounding boxes from circles, [x1 y1 x2 y2]
  for i=1:size(frames,2)
    bb(i,:) = [round(max(1, frames(1,i) - frames(3,i))), ...
               round(max(1, frames(2,i) - frames(3,i))), ...
               round(min(size(I,2), frames(1,i) + frames(3,i))), ...
               round(min(size(I,1), frames(2,i) + frames(3,i)))];
  end
  
  % colect region information
  [h,w,c] = size(I);
  idx = 1;
  for i=1:size(frames,2)
        
    % draw circle on a temporal image
    b = max( [abs(frames(2,i) - frames(3,i)    ) + 1;
              abs(frames(2,i) + frames(3,i) - h) + 1;
              abs(frames(1,i) - frames(3,i)    ) + 1;
              abs(frames(1,i) + frames(3,i) - w) + 1;
              1]);
    b = ceil(b);
    circleImage = zeros(2*b+size(I,1), 2*b+size(I,2));
    circleImage = drawFilledCircleOnImage(circleImage, ...
                                         frames(3,i), ...
                                         frames(2,i)+b, ...
                                         frames(1,i)+b, ...
                                         1);   
    circleImage = circleImage(b+1:b+size(I,1), b+1:b+size(I,2));
        
    % get part of bb that is inside ROI
    x1p = bb(i,1);
    y1p = bb(i,2);
    x2p = bb(i,3);
    y2p = bb(i,4);    
    
    % is some part of the bb inside ROI?
    nPixProp = (x2p-x1p+1) * (y2p-y1p+1);
    if (x2p-x1p >= 0) && (y2p-y1p >= 0) % && nPixProp>100 && nPixProp<(0.5*nPix)
      regions{idx}.ROI = [x1p, y1p, x2p, y2p];
                       
      regions{idx}.patch = I(regions{idx}.ROI(2):regions{idx}.ROI(4), ...
                             regions{idx}.ROI(1):regions{idx}.ROI(3), ...
                             :);
      regions{idx}.mask = circleImage(regions{idx}.ROI(2):regions{idx}.ROI(4), ...
                                      regions{idx}.ROI(1):regions{idx}.ROI(3));
      regions{idx}.label = i; 
      
      regions{idx}.midpoint = [frames(1,i), frames(2,i)];
      
      idx = idx+1;
      
    end
    
  end
 
end